package xyz.karx.spirc;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;

import java.io.*;
import java.net.Socket;
import java.util.function.Supplier;

public class Bot extends Thread {
    private final String USERNAME;
    private final String HOST;
    private final String CHANNEL;
    private final int PORT;

    private Socket s;

    private BufferedWriter bw;
    private BufferedReader br;
    private Chat chat;

    private boolean hasJoined = false;
    private boolean isRunning = false;

    public Bot(String USERNAME, String HOST, String CHANNEL, int PORT) {
        this.USERNAME = USERNAME;
        this.HOST = HOST;
        this.CHANNEL = CHANNEL;
        this.PORT = PORT;
    }

    public void init() {
        try {
            s = new Socket(HOST, PORT);

            bw = new BufferedWriter(new OutputStreamWriter(s.getOutputStream()));
            br = new BufferedReader(new InputStreamReader(s.getInputStream()));
            chat = new Chat(USERNAME, CHANNEL);

            chat.sendToServer(bw, "USER " + USERNAME + " * * :SpIRC");
            chat.sendToServer(bw, "NICK " + USERNAME);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

     public void run() {
        String line = "";
        isRunning = true;


        try {
            while ((line = br.readLine()) != null && isRunning) {
                line = line.replaceAll("\\x03(?:\\d{1,2}(?:,\\d{1,2})?)|[^\\u0020-\\u007E]", "");

                Main.getPlugin(Main.class).getLogger().info(line);

                if (line.contains("PRIVMSG")) {
                    String user = line.substring(1, line.indexOf("!"));
                    String message = line.replaceAll("^[\\w\\W]+ :", "");

                    Bukkit.broadcastMessage(ChatColor.BLUE + "[IRC] <" + user + "> " + ChatColor.WHITE + message);
                } else if (line.contains("PING")) {
                    chat.sendToServer(bw, line.replaceFirst("I", "O"));
                } else if (line.contains("JOIN")) {
                    String user = line.substring(1, line.indexOf("!"));

                    Bukkit.broadcastMessage(ChatColor.YELLOW + "" + ChatColor.ITALIC + "[IRC] " + user + " has joined the channel");
                } else if (line.contains("QUIT") || line.contains("PART")) {
                    String user = line.substring(1, line.indexOf("!"));

                    Bukkit.broadcastMessage(ChatColor.RED + "" + ChatColor.ITALIC + "[IRC] " + user + " has left the channel");
                }

                // This will join the channel once and then not again
                // Won't run until the bot has responded to the first ping (aka registered)
                // if (hasRespondedToFirstPing && !hasJoined) {
                if (line.contains("376") && !hasJoined) {
                    chat.sendToServer(bw, "MODE " + USERNAME + " +B");
                    chat.sendToServer(bw, "JOIN #" + CHANNEL);

                    hasJoined = true;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void terminate() {
        try {
            chat.sendToChat(bw, "Server shutting down!");
            chat.sendToServer(bw, "PART #" + CHANNEL);
            chat.sendToServer(bw, "QUIT :Server shutting down");
            isRunning = false;
            s.close();
            bw.close();
            br.close();
            this.interrupt();
            Main.getPlugin(Main.class).getLogger().info("Thread successfully stopped.");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sendMessage(String message) {
        if (isRunning) {
            chat.sendToChat(bw, message);
        }
    }
}
